import {Metadata, Http, IEventParam, Loader} from '@zeedhi/core';
import { Modal, ModalService, Form, Grid, TextInput, AlertService, DialogService } from '@zeedhi/common';
import { PeriodsService } from '@/Services/PeriodsService';
import {Utilities} from "@/Services/Utilities";
import { CustomAlertService } from '@/Services/CustomAlertService';

export class PeriodsController {
    //
    private addModal !: Modal;
    private viewModal !: Modal;
    private firstTimeSeeingPeriod = true;
    public viewEditDisable = false;
    private currentRow: any = {};
    //
    public reloadGrid() {
        const grid = Metadata.getInstance<Grid>('periods-grid');
        grid.reload();
    }
    //
    public async mountGrid() {
        const appController = Loader.getInstance('AppController');
        await appController.loginPromise.then();
        this.reloadGrid();
    }
    //
    public async openAddModal() {
        if (this.addModal !== undefined) {
            await this.addModal?.destroy();
        }
        this.addModal = await ModalService.createFromJson('periods/addModal/addModal', true);
        this.addModal.show();
    }
    //
    public async closeAddModal() {
        this.addModal.hide();
        await this.addModal?.destroy();
    }
    //
    public async openViewModal() {
        if (this.viewModal !== undefined) {
            await this.viewModal?.destroy();
        }
        this.viewModal = await ModalService.createFromJson('periods/viewModal/viewModal', true);
        this.viewModal.show();
    }
    //
    public async closeViewModal() {
        this.viewModal.hide();
        await this.viewModal?.destroy();
    }
    //
    public async saveAddModal() {
        const form = Metadata.getInstance<Form>("add-form");
        const periodForm = Metadata.getInstance<Form>('period-section-add-form');
        if (!form.validate() || !periodForm.validate()) return;
        const request = {
            dataset: [
                {
                    changed: true,
                    custom: true,
                    __createdLocal: true,
                    __is_new: true,
                    ...form.value,
                    ...periodForm.value
                }
            ],
            disableLoader: false,
            requestType: "DataSet"
        };
        await Http.post('/periods/save', request).then(() => {
            CustomAlertService.saveSuccessAlert();
        });
        this.reloadGrid();
        this.closeAddModal();
    }

    public eventFocusEndDateExpression({component, element}: IEventParam<TextInput>) {
        Utilities.fixBugHintOnfocus(component, element);
    }
    public eventFocusStartDateExpression({component, element}: IEventParam<TextInput>) {
        Utilities.fixBugHintOnfocus(component, element);
    }
    //
    public async saveViewModal() {
        const form = Metadata.getInstance<Form>('view-form');
        const periodForm = Metadata.getInstance<Form>('period-section-view-form');
        const row = this.currentRow;
        if (!form.validate() || !periodForm.validate()) return;
        const request = {
            dataset: [
                {
                    changed: true,
                    custom: true,
                    ...form.value,
                    ...periodForm.value,
                    _id: row._id
                }
            ],
            disableLoader: false,
            requestType: "DataSet"
        };
        await Http.post('/periods/save', request).then(() => {
            CustomAlertService.saveSuccessAlert();
        });
        this.reloadGrid();
        this.closeViewModal();
    }
    //
    private fillForm(form:any, periodForm:any, row:any) {
        form.name = row.name;
        form.labelOnView = row.labelOnView;
        periodForm.START_DATE_EXPRESSION = row.START_DATE_EXPRESSION;
        periodForm.END_DATE_EXPRESSION = row.END_DATE_EXPRESSION;
        periodForm.periodsCode = row.periodsCode;
    }
    //
    public async showRowInfo({component, row} : any) {
        if (component.editing) return;
        await this.openViewModal();
        const form = Metadata.getInstance<Form>('view-form').value;
        const periodForm = Metadata.getInstance<Form>('period-section-view-form').value;
        this.currentRow = row;
        this.fillForm(form, periodForm, row);
        this.viewEditDisable = false;
        if (row.base === true) {
            this.viewEditDisable = true;
            if (this.firstTimeSeeingPeriod) {
                DialogService.show({
                    name: "base-edit-dialog",
                    type: "info",
                    title: "Atenção",
                    text: "Não é possível editar períodos do sistema."
                });
                this.firstTimeSeeingPeriod = false;
            }
        }
    }
    //
    public showConfirmDelete({event} : any) {
        event.preventDefault();
        DialogService.show({
            name: 'confirm-delete',
            type: 'warning',
            title: ' ',
            text: 'Deseja remover o(s) registro(s) selecionado(s)?',
            buttons: [
                { name: 'nao', component: 'ZdButton', label: 'Não', events: { click: () => { DialogService.hide(); } }, outline: true },
                { name: 'sim', component: 'ZdButton', label: 'Sim', events: { click: () => { this.deleteRow(); DialogService.hide(); } } },
            ],
        });
    }
    //
    private async deleteRow() {
        const grid = Metadata.getInstance<Grid>('periods-grid');
        for (let i = 0; i < grid.selectedRows.length; i += 1) {
            if (grid.selectedRows[i].base === true) {
                CustomAlertService.errorAlert("Não é possível deletar período(s) do sistema!");
                return;
            }
        }
        const data = {
            "dataset": grid.selectedRows,
            "disableLoader": false,
            'requestType': 'DataSet'
        };
        await Http.post('/periods/remove', data).then(() => {
            CustomAlertService.deleteSuccessAlert();
        });
        this.reloadGrid();
    }
    //
    public testPeriod({component} : any) {
        const periodForm = Metadata.getInstance<Form>('period-section-view-form').value;
        let periodCode = periodForm.periodsCode;
        const startDateExpression = periodForm.START_DATE_EXPRESSION;
        const endDateExpression = periodForm.END_DATE_EXPRESSION;
        const periodsService = new PeriodsService();
        try {
            if (startDateExpression || endDateExpression || !periodCode) {
                periodsService.getDateByExpressions([startDateExpression, endDateExpression]).then((dates) => {
                    DialogService.show({
                        name: "base-edit-dialog",
                        type: "info",
                        title: "Período",
                        text: "De " + dates[0] + " a " + dates[1]
                    });
                    this.firstTimeSeeingPeriod = false;
                }).catch((err) => { console.error(err); });
            } else {
                periodCode = periodsService.conditionPeriod(periodCode, false);
                const interval = periodsService.decodePeriod(periodCode);
                DialogService.show({
                    name: "base-edit-dialog",
                    type: "info",
                    title: "Período",
                    text: "De " + interval.start + " a " + interval.end
                });
            }
        } catch (error) {
            DialogService.show({
                name: "error-date-dialog",
                type: "error",
                title: error.name,
                // "Período Incorreto"
                text: error.message
                // "Período utilizado está em um formato incorreto."
            });
        }
    }
    //
    public isRowEditable({row} : any) {
        return !row.base;
    }
    //
    public async savePeriodInline({component} : any) {
        const rows = component.getEditedRows().map((row: any) => ({changed: true, custom: true, ...row}));
        if (rows.length === 0) {
            return;
        }
        const request = {
            dataset: rows,
            disableLoader: false,
            requestType: "DataSet"
        };
        await Http.post('/periods/save', request).then(() => {
            CustomAlertService.saveSuccessAlert();
        });
        this.reloadGrid();
    }
}
